#!/bin/bash
###############################################################################
# Brief     : 守护启动指定命令的进程
# Usage     ：./daemon.sh <command> <interval(10 seconds)> <output>
#           : command   - 命令，可包含命令参数
#           : interval  - 守护进程检查时间间隔，默认10秒
#           : output    - 指定命令进程终端输出重定向文件
# Version   : 1.1
# Date      : 20190327
# 20190515 修改程序启动函数为进入程序所做目录启动,以解决需要加载程序所在目录的so而造成的启动错误;
###############################################################################

work_dir=$(cd $(dirname ${BASH_SOURCE[0]}); pwd)

function logfunc (){
    echo -e "$(date '+%Y%m%d-%H:%M:%S') $1"
}

# 启动程序
function prog_start() {
    local command=$1
    #crontab如果不能启动,可以设置export或者在启动脚本中source ~/.bash_profile
    source ~/.bash_profile
    export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${work_dir}:$(cd $(dirname ${command}); pwd)
    echo -e "the process [ ${command} ] start success!"
    cd $(dirname ${command})
    nohup "${command}" ./config.xml >/dev/null 2>&1 &
}

# 关闭程序
function prog_stop() {
    local command=$1
    [ -z "$command" ] && echo -e "command is empty" && return 1
    # 不关闭守护进程
    local num=`ps -ef | grep -v grep | grep -v daemon | grep -w "${command}" | wc -l`
    if [ "${num}" -gt "0" ]; then
        ps -ef | grep -v grep | grep -v daemon | grep -w "${command}" | awk '{print $2}' | xargs kill -9 >/dev/null 2>&1
        local tt=`ps -ef | grep -v grep | grep -v daemon | grep -w "${command}" | wc -l`
        if [ ${tt} -eq 0 ]; then
            echo -e "the process [ ${command} ] stopped success!"
        else
            echo -e "the process [ ${command} ] stopped failed!"
        fi
    fi
}

function run_daemon() {
    local command=$1
    [ -z "${command}" ] && logfunc "command is empty" && return 1
    local interval=$2
    [ -z "${interval}" ] && interval="10s"
    local output=$3
    [ -z "${output}" ] && output="${work_dir}/output$(date '+%Y%m%d').log"
    logfunc "daemon for [ ${command} ] is started." >> ${output}
    while true; do
        #会导致自定义的第三个参数无效
        output="${work_dir}/output$(date '+%Y%m%d').log"
        find ${work_dir}/output*.log -type f -mtime +15 -exec rm -fv {} \; >> ${output} 2>&1  # 删除15天前的日志
        # prog's num
        local num=`ps -ef | grep -v grep | grep -v daemon | grep "${command}" | wc -l`
        if [ ${num} -lt 1 ]; then
            logfunc "the process: [ ${command} ] is not running, start" >> ${output}
            #进入程序所在目录,如果错误则退出本次if循环
            cd $(dirname ${command}) && prog_start "${command}" >> ${output} && cd ${work_dir}         
        elif [ ${num} -eq 1 ]; then
            local process_pid=`ps -ef | grep -v grep | grep -v daemon | grep "${command}" | awk '{print $2}'`
            logfunc "the process: [ ${command} ] is still running, num is: [ ${num} ], pid is:[ ${process_pid} ]" >> ${output}
        elif [ ${num} -gt 1 ]; then
            local process_pids=`ps -ef | grep -v grep | grep -v daemon | grep "${command}" | awk '{print $2}'`
            logfunc "warning: [ ${num} ] processes of command [ ${command} ] are running pids are :[ ${process_pids} ], restart" >> ${output}
            prog_stop "${command}" >> ${output}
            cd $(dirname ${command}) && prog_start "${command}" >> ${output} && cd ${work_dir}
        fi
        sleep "$interval"
    done
}


daemon_output="${work_dir}/output$(date '+%Y%m%d').log"
run_daemon "$1" "$2" "$3" >> "${daemon_output}" 2>&1 &
